@@ -33,6 +33,7 @@ import pprint as pp
 import pstats
 import shlex
 import shutil
+import signal
 import subprocess
 import sys
 import tempfile
@@ -724,9 +725,18 @@ def alert(title, content, action='Close'):
     subprocess.run(cmd)
 
 
-def convert_to_wine_path(path, drive='Z:'):
-    path = abspath(path).replace('/', '\\')
-    return drive + path
+def convert_to_wine_path(path, drive=None):
+    full_path = abspath(expanduser(path))
+    home_folder = os.environ['HOME']
+    if leading_homefolder := full_path.startswith(home_folder):
+        mapped_drive = 'Y:'
+        full_path = full_path.removeprefix(home_folder)
+    else:
+        mapped_drive = 'Z:'
+    if drive:
+        mapped_drive = drive
+    full_path = full_path.replace('/', '\\')
+    return mapped_drive + full_path
 
 
 def convert_from_wine_path(path):
@@ -782,30 +792,63 @@ class RerunLock:
         os.makedirs(folder, exist_ok=True)
         filename = f'lock_{name}.json' if name else 'lock_{}.json'.format(next(tempfile._get_candidate_names()))
         self.lockFile = osp.join(folder, filename) if folder else join(get_local_tmp_dir(), filename)
-        self.infoHook = logger.info
-        self.warnHook = logger.warning
-        self.errorHook = logger.error
+        self.logger = logger
+        common_sigs = [
+            signal.SIGABRT,
+            signal.SIGFPE,
+            signal.SIGILL,
+            signal.SIGINT,
+            signal.SIGSEGV,
+            signal.SIGTERM,
+        ]
+        plat_sigs = [
+            signal.SIGBREAK,
+            # CAUTION
+            # - CTRL_C_EVENT, CTRL_BREAK_EVENT not working on Windows
+            # signal.CTRL_C_EVENT,
+            # signal.CTRL_BREAK_EVENT,
+        ] if platform.system() == 'Windows' else [
+            # CAUTION:
+            # - SIGCHLD as an alias is safe to ignore
+            # - SIGKILL must be handled by os.kill()
+            signal.SIGALRM,
+            signal.SIGBUS,
+            # signal.SIGCHLD,
+            signal.SIGCONT,
+            signal.SIGHUP,
+            # signal.SIGKILL,
+            signal.SIGPIPE,
+        ]
+        for sig in common_sigs + plat_sigs:
+            # breakpoint()
+            signal.signal(sig, self.handle_signal)
 
     def lock(self):
         if not self.is_locked():
             save_json(self.lockFile, {'pid': os.getpid()})
             return True
         else:
-            self.warnHook('Locked by pid: {}. Will stay locked until it ends.'.format(os.getpid()))
+            self.logger.warning('Locked by pid: {}. Will stay locked until it ends.'.format(os.getpid()))
             return False
 
     def unlock(self):
         try:
             os.remove(self.lockFile)
         except FileNotFoundError:
-            self.warnHook('Already unlocked. Aborted.')
+            self.logger.warning('Already unlocked. Aborted.')
         except Exception:
             failure = traceback.format_exc()
-            self.errorHook('{}\nFailed to unlock. Must delete the lock by hand: {}.'.format(failure, self.lockFile))
+            self.logger.error('{}\nFailed to unlock. Must delete the lock by hand: {}.'.format(failure, self.lockFile))
 
     def is_locked(self):
         return osp.isfile(self.lockFile)
 
+    def handle_signal(self, sig, frame):
+        msg = f'Terminated due to signal: {signal.Signals(sig).name}; Will unlock'
+        self.logger.warning(msg)
+        self.unlock()
+        raise RuntimeError(msg)
+
 
 def rerun_lock(name, folder=None, logger=_logger):
     """Decorator for reentrance locking on functions"""
@@ -819,7 +862,7 @@ def rerun_lock(name, folder=None, logger=_logger):
                     return 1
                 ret = f(*args, **kwargs)
                 my_lock.unlock()
-            except Exception as e:  
+            except Exception as e:
                 my_lock.unlock()
                 # leave exception to its upper handler or let the program crash
                 raise e
@@ -924,14 +967,14 @@ def remove_from_os_paths(bindir, usesyspath=True):
     os.environ[path_var] = os.environ[path_var].replace(bindir, '')
 
 
-def run_cmd(cmd, cwd='.', logger=None):
+def run_cmd(cmd, cwd='.', logger=None, check=True):
     if logger:
         logger.debug(' '.join(cmd))
     else:
         print(' '.join(cmd))
     proc = None
     try:
-        proc = subprocess.run(cmd, check=True, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=cwd)
+        proc = subprocess.run(cmd, check=check, shell=False, stdout=subprocess.PIPE, stderr=subprocess.PIPE, cwd=cwd)
         log = proc.stdout.decode(TXT_CODEC)
         if logger:
             logger.debug(log)
@@ -1315,6 +1358,7 @@ def compare_textfiles(file1, file2, showdiff=False, contextonly=True, logger=Non
         if showdiff:
             diff_func = difflib.context_diff if contextonly else difflib.Differ().compare
             diff = diff_func(lines1, lines2)
+            lazy_logging(f'*** {file1} vs. {file2} ***')
             lazy_logging(''.join(diff), logger)
     return lines1 == lines2
 
@@ -1420,7 +1464,7 @@ def get_parent_dirs(file, subs=(), depth=1):
 
 
 def get_child_dirs(root, subs=()):
-    return root, *[osp.join(root, sub) for sub in subs]
+    return (osp.join(root, sub) for sub in subs)
 
 
 def read_lines(file, striplineend=False, posix=True):
@@ -1433,6 +1477,28 @@ def read_lines(file, striplineend=False, posix=True):
     return lines
 
 
+def open_in_browser(path, window='tab', islocal=True):
+    import webbrowser as wb
+    url = f'file://{path}' if islocal else path
+    api = {
+        'current': wb.open,
+        'tab': wb.open_new_tab,
+        'window': wb.open_new,
+    }
+    api[window](url)
+
+
+def open_in_editor(path):
+    cmds = {
+        'Windows': 'explorer',
+        'Darwin': 'open',
+        'Linux': 'xdg-open',  # ubuntu
+    }
+    cmd = [cmds[platform.system()], path]
+    check = platform.system() != 'Windows'
+    run_cmd(cmd, check=check)
+
+
 def _test():
     pass
 
